import { Operator } from "@reactive-js/pipe";
import { HttpMethod, URILike, HttpHeaders, HttpRequest, HttpResponse, HttpServerRequest, CacheDirective, EntityTag, HttpContentEncoding, MediaRange, MediaType } from "./interfaces";
export declare const createHttpRequest: <T>({ body, cacheControl, contentInfo, expectContinue, headers, httpVersionMajor, httpVersionMinor, method, preconditions, preferences, uri, ...rest }: {
    body: T;
    cacheControl?: readonly (string | CacheDirective)[] | undefined;
    contentInfo?: {
        contentEncodings?: HttpContentEncoding[] | undefined;
        contentLength?: number | undefined;
        contentType: string | MediaType;
    } | undefined;
    expectContinue?: boolean | undefined;
    headers?: HttpHeaders | undefined;
    httpVersionMajor?: number | undefined;
    httpVersionMinor?: number | undefined;
    method: HttpMethod;
    preconditions?: {
        ifMatch?: "*" | (string | EntityTag)[] | undefined;
        ifModifiedSince?: string | number | Date | undefined;
        ifNoneMatch?: "*" | (string | EntityTag)[] | undefined;
        ifUnmodifiedSince?: string | number | Date | undefined;
        ifRange?: string | number | EntityTag | Date | undefined;
    } | undefined;
    preferences?: {
        acceptedCharsets?: readonly string[] | undefined;
        acceptedEncodings?: readonly HttpContentEncoding[] | undefined;
        acceptedLanguages?: readonly string[] | undefined;
        acceptedMediaRanges?: readonly (string | MediaRange)[] | undefined;
    } | undefined;
    uri: string | URILike;
}) => HttpRequest<T>;
export declare const createRedirectHttpRequest: <THttpRequest extends HttpRequest<TReq>, TReq>(request: THttpRequest, response: HttpResponse<unknown>) => THttpRequest;
export declare const parseHttpRequestFromHeaders: <T>({ method, path, headers, httpVersionMajor, httpVersionMinor, isTransportSecure, body, }: {
    method: HttpMethod;
    path: string;
    headers: HttpHeaders;
    body: T;
    httpVersionMajor: number;
    httpVersionMinor: number;
    isTransportSecure: boolean;
}) => HttpServerRequest<T>;
export declare const writeHttpRequestHeaders: <T>(request: HttpRequest<T>, writeHeader: (header: string, value: string) => void) => void;
export declare const disallowProtocolAndHostForwarding: <T>() => Operator<HttpServerRequest<T>, HttpServerRequest<T>>;
export declare const httpRequestToUntypedHeaders: (request: HttpRequest<unknown>) => {
    [key: string]: string;
};
export declare const httpRequestIsCompressible: <T>({ contentInfo }: HttpRequest<T>, db: {
    [key: string]: {
        compressible?: boolean | undefined;
    };
}) => boolean;
