import { __rest } from "tslib";
import { isNone, isSome, none } from "@reactive-js/option";
import { parseHttpContentInfoFromHeaders, contentIsCompressible, createHttpContentInfo, } from "./httpContentInfo";
import { getHeaderValue, filterHeaders, } from "./httpHeaders";
import { parseHttpPreferencesFromHeaders, createHttpPreferences, } from "./httpPreferences";
import { writeHttpRequestPreconditionsHeaders, parseHttpRequestPreconditionsFromHeaders, createHttpRequestPreconditions, } from "./httpRequestPreconditions";
import { parseCacheControlFromHeaders, parseCacheDirectiveOrThrow, } from "./cacheDirective";
import { writeHttpMessageHeaders } from "./HttpMessage";
export const createHttpRequest = (_a) => {
    var { body, cacheControl, contentInfo, expectContinue, headers, httpVersionMajor, httpVersionMinor, method, preconditions, preferences, uri } = _a, rest = __rest(_a, ["body", "cacheControl", "contentInfo", "expectContinue", "headers", "httpVersionMajor", "httpVersionMinor", "method", "preconditions", "preferences", "uri"]);
    return (Object.assign(Object.assign({}, rest), { body, cacheControl: (cacheControl !== null && cacheControl !== void 0 ? cacheControl : []).map(cc => typeof cc === "string" ? parseCacheDirectiveOrThrow(cc) : cc), contentInfo: isSome(contentInfo) ? createHttpContentInfo(contentInfo) : none, expectContinue: expectContinue !== null && expectContinue !== void 0 ? expectContinue : false, headers: filterHeaders(headers !== null && headers !== void 0 ? headers : {}), httpVersionMajor: httpVersionMajor !== null && httpVersionMajor !== void 0 ? httpVersionMajor : 1, httpVersionMinor: httpVersionMinor !== null && httpVersionMinor !== void 0 ? httpVersionMinor : 1, method, preconditions: isSome(preconditions)
            ? createHttpRequestPreconditions(preconditions)
            : none, preferences: isSome(preferences)
            ? createHttpPreferences(preferences)
            : none, uri: typeof uri === "string" ? new URL(uri) : uri }));
};
export const createRedirectHttpRequest = (request, response) => {
    const { contentInfo, method } = request;
    const { location, statusCode } = response;
    const redirectToGet = statusCode === 303 ||
        ((statusCode === 301 ||
            302 === 302) &&
            method === "POST");
    return Object.assign(Object.assign({}, request), { content: redirectToGet ? none : contentInfo, method: redirectToGet ? "GET" : method, uri: location });
};
const parseURIFromHeaders = (protocol, path, httpVersionMajor, headers) => {
    const forwardedProtocol = getHeaderValue(headers, "X-Forwarded-Proto");
    const uriProtocol = isSome(forwardedProtocol)
        ? forwardedProtocol.split(/\s*,\s*/, 1)[0]
        : protocol;
    const forwardedHost = getHeaderValue(headers, "X-Forwarded-Host");
    const http2Authority = headers[":authority"];
    const http1Host = getHeaderValue(headers, "Host");
    const unfilteredHost = isSome(forwardedHost)
        ? forwardedHost
        : isSome(http2Authority) && httpVersionMajor >= 2
            ? http2Authority
            : isSome(http1Host)
                ? http1Host
                : "";
    const host = unfilteredHost.split(/\s*,\s*/, 1)[0];
    return new URL(`${uriProtocol}://${host}${path !== null && path !== void 0 ? path : ""}`);
};
export const parseHttpRequestFromHeaders = ({ method, path, headers, httpVersionMajor, httpVersionMinor, isTransportSecure, body, }) => {
    const cacheControl = parseCacheControlFromHeaders(headers);
    const contentInfo = parseHttpContentInfoFromHeaders(headers);
    const rawExpectHeader = getHeaderValue(headers, "Expect");
    const expectContinue = rawExpectHeader === "100-continue";
    const preconditions = parseHttpRequestPreconditionsFromHeaders(headers);
    const preferences = parseHttpPreferencesFromHeaders(headers);
    const protocol = isTransportSecure ? "https" : "http";
    const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
    return {
        body,
        cacheControl,
        contentInfo,
        expectContinue,
        headers,
        httpVersionMajor,
        httpVersionMinor,
        isTransportSecure,
        method,
        preconditions,
        preferences,
        uri,
    };
};
export const writeHttpRequestHeaders = (request, writeHeader) => {
    const { expectContinue, preconditions, } = request;
    if (expectContinue) {
        writeHeader("Expect", "100-continue");
    }
    if (isSome(preconditions)) {
        writeHttpRequestPreconditionsHeaders(preconditions, writeHeader);
    }
    writeHttpMessageHeaders(request, writeHeader);
};
export const disallowProtocolAndHostForwarding = () => request => {
    const { httpVersionMajor, headers: oldHeaders, isTransportSecure, uri: oldUri, } = request;
    const { "x-forwarded-proto": xForwardedProto, "x-forwarded-host": xForwardedHost } = oldHeaders, headers = __rest(oldHeaders, ["x-forwarded-proto", "x-forwarded-host"]);
    const protocol = isTransportSecure ? "https" : "http";
    if (isNone(xForwardedProto) && isNone(xForwardedHost)) {
        return request;
    }
    else {
        const path = oldUri.pathname;
        const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
        return Object.assign(Object.assign({}, request), { uri,
            headers });
    }
};
export const httpRequestToUntypedHeaders = (request) => {
    const headers = {};
    writeHttpRequestHeaders(request, (header, value) => (headers[header] = value));
    return headers;
};
export const httpRequestIsCompressible = ({ contentInfo }, db) => isSome(contentInfo) && contentIsCompressible(contentInfo, db);
//# sourceMappingURL=httpRequest.js.map